@@ -1,4 +1,4 @@
-import os
+import os, sys
 import numpy as np
 import scipy.optimize as optimize
 import PyAstronomy.pyasl as pyasl
@@ -39,6 +39,24 @@ def dateandtime2mjd(date=None, time='12:00', to_tai=7):
     return(times.mjd)
 
 
+def nExpPrioritize(nexp, base=20, award=2e6, penalty=-100):
+    """adjust field priorities based on planned exps
+
+       1 exp: negative, 8 exp: super high
+    """
+
+    assert nexp < 9 and nexp > 0, "invalid nexp"
+
+    if nexp == 1:
+        return penalty
+    elif nexp == 8:
+        return award
+    elif nexp == 2:
+        return 0
+    else:
+        return nexp * base
+
+
 class SchedulerBase(object):
     """Scheduler base class with generic utilities.
 
@@ -796,16 +814,30 @@ class Scheduler(Master):
     """
     def __init__(self, airmass_limit=2.,
                  schedule='normal', observatory='apo', observatoryfile=None,
-                 exp_time=None):
+                 exp_time=None, priorities={}):
         """Return Scheduler object
         """
         super().__init__(schedule=schedule, observatory=observatory,
                          observatoryfile=observatoryfile)
+        self.check = False
+        self.recent = list()
+        self.recent_ids = list()
         self.airmass_limit = airmass_limit
         if exp_time is None:
             self.exp_time = 18 / 60 / 24
         else:
             self.exp_time = exp_time
+
+        # priorities
+        self.priorities = priorities  # need it later
+        self.nepochsPri = priorities.get("nepochsPri", 10)
+        self.nExpPriBase = priorities.get("nExpPriBase", 20)
+        self.nExpPriAward = priorities.get("nExpPriAward", 2e6)
+        self.nExpPriPenalty = priorities.get("nExpPriPenalty", -100)
+        self.basePri = priorities.get("basePri", 100)
+        self.lstPri = priorities.get("lstPri", 100)
+        self.overheadPri = priorities.get("overheadPri", 40)
+
         return
 
     def initdb(self, designbase='plan-0', fromFits=True):
@@ -832,9 +864,10 @@ class Scheduler(Master):
             cadence_file = filebase + "/" + "rsCadences" + "-" + designbase + "-"\
                            + self.observatory + ".fits"
             fields_file = filebase + "/" + "rsAllocation" + "-" + designbase + "-"\
-                           + self.observatory + ".fits"
+                          + self.observatory + ".fits"
 
-            self.cadencelist.fromfits(filename=cadence_file)
+            self.cadencelist.fromfits(filename=cadence_file,
+                                      priorities=self.priorities)
             self.fields.fromfits(filename=fields_file)
         else:
             self.cadencelist.fromdb()
@@ -884,6 +917,7 @@ class Scheduler(Master):
                                    dec=self.fields.deccen)
 
         # whereRM = np.where(["bhm_rm" in c for c in self.fields.cadence])[0]
+        where_uhoh = np.where(["dark_2x" in c for c in self.fields.cadence])[0]
 
         next_change, next_brightness = self.next_change(mjd)
 
@@ -893,6 +927,11 @@ class Scheduler(Master):
             if skybrightness <= 0.35:
                 # it's still dark
                 nexp_change = 1e3
+            elif nexp_change == 0:
+                # going into dark from bright, don't waste 15 minutes?
+                # delta V check will cover bad cases
+                skybrightness = 0.34
+                nexp_change = maxExp
         else:
             # next is bright
             if skybrightness > 0.35:
@@ -905,6 +944,8 @@ class Scheduler(Master):
 
         # print(f"{float(mjd):.3f} {float(next_change):.3f} {float(next_brightness):.2f} {nexp_change}", maxExp)
 
+        over = list()
+
         # indxs = np.where(self.fields.nextmjd > mjd)[0]
         # observable[indxs] = False
         indxs = np.where(observable)[0]
@@ -923,24 +964,49 @@ class Scheduler(Master):
             # for 0 indexed arrays, this equivalent to
             # "how many epochs have I done previously"
             tol = cadence.max_length[int(self.fields.epoch_idx[indx])]
-            epoch_idx, mjd_prev = epochs_completed(mjd_past, tolerance=tol)
+            if tol < 0.1:
+                tol = 0.1
+            elif tol > 14:
+                tol = 14
+            epoch_idx, begin_last_epoch = epochs_completed(mjd_past, tolerance=tol)
             # how many exp/"designs" since start of last epoch?
-            exp_epoch = np.sum(np.greater(mjd_past, mjd_prev))
+            exp_epoch = np.sum(np.greater_equal(mjd_past, begin_last_epoch))
+            if len(mjd_past):
+                mjd_prev = mjd_past[-1]
+            else:
+                mjd_prev = 0
+            if len(mjd_past) > np.sum(cadence.nexp):
+                over.append(int(self.fields.field_id[indx]))
+            actually_done = False
+            # because dark-2xN can have max length 7 but delta min 0.5
+            while exp_epoch > cadence.nexp[epoch_idx]:
+                exp_epoch -= cadence.nexp[epoch_idx]
+                epoch_idx += 1
+                if epoch_idx == cadence.nepochs:
+                    actually_done = True
+                    break
+            if actually_done:
+                observable[indx] = False
+                continue
             if exp_epoch < cadence.nexp[epoch_idx] and exp_epoch != 0:
                 nexp[indx] = cadence.nexp[epoch_idx] - exp_epoch
                 epoch_idx -= 1
                 partial_epoch = True
+            if epoch_idx >= cadence.nepochs and not self.fields.flag[indx] == 1:
+                # print("DONE ", epoch_idx, cadence.nepochs, int(self.fields.field_id[indx]))
+                observable[indx] = False
+                continue
             else:
                 nexp[indx] = cadence.nexp[epoch_idx]
                 partial_epoch = False
-            if epoch_idx >= cadence.nepochs:
-                print("DONE ", epoch_idx, cadence.nepochs, int(self.fields.field_id[indx]))
-                observable[indx] = False
-                continue
             if nexp[indx] > nexp_change and self.fields.flag[indx] != 1:
+                # print(f"{float(mjd):.3f} {int(self.fields.field_id[indx])} c_nexp {cadence.nexp[epoch_idx]} NEXP {nexp_change} MAXEXP {maxExp}")
                 # change between bright/dark, field doesn't fit
                 observable[indx] = False
                 continue
+            # if int(self.fields.field_id[indx]) in over:
+            #     print("!?", epoch_idx >= cadence.nepochs, not self.fields.flag[indx] == 1)
+            #     print(int(self.fields.field_id[indx]), len(mjd_past), cadence.nexp, exp_epoch, cadence.nepochs, epoch_idx)
             observable[indx], delta_priority[indx] =\
                 cadence.evaluate_next(epoch_idx=epoch_idx,
                                       partial_epoch=partial_epoch,
@@ -952,8 +1018,16 @@ class Scheduler(Master):
                                       airmass=airmass[indx])
             # if nexp[indx] > 6 and observable[indx]:
             #     delta_priority[indx] += 1e6
-            delta_priority[indx] += nexp[indx] * 10
+            # delta_priority[indx] += 2.5**nexp[indx]
+            delta_priority[indx] += nExpPrioritize(nexp[indx],
+                                                   base=self.nExpPriBase,
+                                                   award=self.nExpPriAward,
+                                                   penalty=self.nExpPriPenalty)
+            # prioritize nepochs. Huge bump for 8 + epochs
+            # slight decrement for single epochs
+            delta_priority[indx] += (cadence.nepochs - 2)*self.nepochsPri
             if nexp[indx] > maxExp:
+                # print(f"{float(mjd):.3f} {int(self.fields.field_id[indx])} c_nexp {cadence.nexp[epoch_idx]} MAXEXP {maxExp}")
                 observable[indx] = False
             if self.fields.flag[indx] == 1:
                 # flagged as top priority
@@ -962,9 +1036,26 @@ class Scheduler(Master):
                 # so override cadence eligibility and bump priority
                 observable[indx] = True
                 delta_priority[indx] += 1e6
+            # if "dark_2x2" in cadence.name and observable[indx] and mjd_prev != 0:
+            #     self.check = True
+            #     # print(f"F {float(mjd):.3f} {int(self.fields.field_id[indx])} {delta_priority[indx]}")
+            #     self.recent.append(f"R {int(self.fields.field_id[indx])} {epoch_idx} prev {len(mjd_past)} epoch {epoch_idx} tol {tol} partial {partial_epoch}")
+            #     self.recent_ids.append(int(self.fields.field_id[indx]))
+                # print(nexp[indx], type(nexp[indx]))
+            # elif "x8" in cadence.name and not observable[indx]:
+            #     if skybrightness < 0.36 and moon_dist[indx] < 30 and deltav[indx] < 2 and airmass[indx] < 1.5:
+            #         print(f"F {epoch_idx} {float(mjd):.3f} {int(self.fields.field_id[indx])} {mjd-mjd_prev}")
+            #         print(f"sky {skybrightness} moon {moon_dist[indx]} deltav {deltav[indx]} AM {airmass[indx]:}")
+            #         print(nexp[indx], type(nexp[indx]), maxExp)
         # print(f"{float(mjd):.3f} {float(next_change):.3f} {float(next_brightness):.2f} {nexp_change}", maxExp)
         iobservable = np.where(observable)[0]
 
+        # if len(iobservable) == 0 and skybrightness < 0.35:
+        #     for i, v, a in zip(self.fields.field_id[where_uhoh], deltav[where_uhoh], airmass[where_uhoh]):
+        #         print(int(i), v, a)
+        #     print("failed", mjd, next_change, maxExp)
+        #     sys.exit(1)
+
         return iobservable, nexp[iobservable], delta_priority[iobservable]
 
     def prioritize(self, mjd=None, iobservable=None, nexp=None,
@@ -995,11 +1086,16 @@ class Scheduler(Master):
             calculated priority for each field
         """
 
-        priority = np.ones(len(iobservable))*100
+        priority = np.ones(len(iobservable)) * self.basePri
         # priority = self.fields.basePriority[fieldid]
         priority += delta_priority
 
-        priority += np.power(2, nexp) * 5  # 1280 for RM
+        # # #################
+        # # #################
+        # # GRRRRRR
+        # # #################
+        # # #################
+        # priority += np.power(2, nexp) * 5  # 1280 for RM
 
         lst = self.lst(mjd)
 
@@ -1016,11 +1112,11 @@ class Scheduler(Master):
         dec = self.fields.deccen[iobservable]
 
         # gaussian weight, mean already 0, use 1 hr  std
-        priority += 50 * np.exp(-(lstDiffs)**2 / (2 * 0.5**2))
+        priority += self.lstPri * np.exp(-(lstDiffs)**2 / (2 * 0.5**2))
         # gaussian weight, mean already 0, use 1 hr = 15 deg std
-        priority += 50 * np.exp(-(ha)**2 / (2 * 15**2))
+        # priority += 50 * np.exp(-(ha)**2 / (2 * 15**2))
         # gaussian weight, mean = obs lat, use 20 deg std
-        priority -= 20 * np.exp(-(dec - self.latitude)**2 / (2 * 20**2))
+        priority -= self.overheadPri * np.exp(-(dec - self.latitude)**2 / (2 * 20**2))
 
         return priority
 
@@ -1052,10 +1148,12 @@ class Scheduler(Master):
         pick_fieldid = fieldid[ipick]
         pick_exp = nexp[ipick]
 
-        # if self.check:
-        #     self.chosen[-1] = pick_fieldid
-        #     self.chosen_pri[-1] = float(priority[ipick])
-        #     print("CHOICE", pick_fieldid, np.max(priority), ipick, pick_exp, "\n")
+        # if self.check and int(pick_fieldid) in self.recent_ids:
+        #     # self.chosen[-1] = pick_fieldid
+        #     # self.chosen_pri[-1] = float(priority[ipick])
+        #     print("\n CHOICE", pick_fieldid, np.max(priority), pick_exp)
+        #     for r in self.recent:
+        #         print(r)
 
         return(pick_fieldid, pick_exp)
 
@@ -1127,6 +1225,9 @@ class Scheduler(Master):
         sorted_exp : list of integer
             a list of nexp sorted by priority
         """
+        self.check = False
+        self.recent = list()
+        self.recent_ids = list()
 
         iobservable, nexp, delta_priority = self.observable(mjd=mjd, maxExp=maxExp,
                                                             ignore=ignore)
